/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.handler;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.resource.ImageDescriptor;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.base.ModelElementHandlerBase;
import org.fortiss.tooling.kernel.interfaces.IModelElementHandler;
import org.fortiss.tooling.kernel.model.IProjectRootElement;
import org.unicase.ecp.model.workSpaceModel.ECPProject;
import org.unicase.ecpemfstorebridge.EMFStoreECPProject;
import org.unicase.workspace.ProjectSpace;

/**
 * {@link IModelElementHandler} for {@link ECPProject}s.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public final class EMFStoreECPProjectModelElementHandler extends
		ModelElementHandlerBase<EMFStoreECPProject> {

	/** {@inheritDoc} */
	@Override
	public String getName(EMFStoreECPProject element) {
		ProjectSpace projectSpace = (ProjectSpace) element.getRootObject();
		String projectName = projectSpace.getProjectName();
		String baseRevision = " @"
				+ projectSpace.getBaseVersion().getIdentifier();
		String username = projectSpace.getUsersession().getUsername();
		return projectName + baseRevision + " (" + username + ")";
	}

	/** {@inheritDoc} */
	@Override
	public String getDescription(EMFStoreECPProject element) {
		ProjectSpace projectSpace = (ProjectSpace) element.getRootObject();
		return projectSpace.getProjectDescription();
	}

	/** {@inheritDoc} */
	@Override
	public ImageDescriptor getIconImageDescriptor() {
		return ToolingKernelActivator.getImageDescriptor("icons/project.png");
	}

	/** {@inheritDoc} */
	@Override
	public List<EObject> getSubnodes(EMFStoreECPProject element) {
		List<EObject> list = new ArrayList<EObject>();
		for (EObject node : element.getAllModelElements()) {
			if (node instanceof IProjectRootElement) {
				list.add(node);
			}
		}
		return list;
	}
}
